module PaymentsjsRails
	require 'paymentsjs-rails/engine'
end

class PaymentsJs
	require 'helpers/configuration'
	require 'openssl'
	require 'base64'
	require 'json'
	
	extend Configuration
	
	# Generate a salt 
	def generate_salt(iv)
		salt = iv.unpack('H*').first
		salt = salt.bytes.to_a
		salt = salt.pack('U*')
		Base64.strict_encode64(salt)
	end
	
	# Set defaults for testing, these will be overwritten in the config
	define_setting :request_type, "payment"
	define_setting :pre_auth, false
	define_setting :postback_url, "https://www.example.com"
	define_setting :environment, "cert"
	
	attr_accessor :address, :city, :state, :zip, :amount, :request_id, :name, :request_type, :pre_auth, :environment, :api_key, :salt, :mid, :postback_url, :auth_key

	def initialize(args = {})

		@api_key      = PaymentsJs.api_key
		@mid          = PaymentsJs.mid
		@mkey         = PaymentsJs.mkey
		@api_secret   = PaymentsJs.api_secret

		@request_type = args[:request_type].present? ? args[:request_type] : PaymentsJs.request_type
		@request_id   = args[:request_id]
		@postback_url = args[:postback_url].present? ? args[:postback_url] : PaymentsJs.postback_url
		@amount       = args[:amount]
		@pre_auth     = args[:pre_auth].present? ? args[:pre_auth] : PaymentsJs.pre_auth
		@environment  = args[:environment].present? ? args[:environment] : PaymentsJs.environment

		# Generate the salt and iv at initialization so they can be consistently called
		@iv = OpenSSL::Random.pseudo_bytes(16)
		@salt = generate_salt(@iv)

	end

	def get_auth_key
		cipher = OpenSSL::Cipher::AES.new(256, :CBC)
		cipher.encrypt

		req  = {
			"apiKey"      => @api_key,
			"merchantId"  => @mid,
			"merchantKey" => @mkey,
			"requestType" => @request_type,
			"requestId"   => @request_id,
			"postbackUrl" => @postback_url,
			"amount"      => @amount,
			"nonce"       => @salt,
			"preAuth"     => @pre_auth,
			"environment" => @environment
		}

		data       = JSON.generate(req)
		key        = OpenSSL::PKCS5.pbkdf2_hmac_sha1(@api_secret, @salt, 1500, 32)
		cipher.key = key
		cipher.iv  = @iv
		auth_key    = cipher.update(data) + cipher.final()

		Base64.strict_encode64(auth_key)
	end

	def get_salt
		@salt
	end
end
